from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class Ping(_message.Message):
    __slots__ = ("count",)
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...

class Pong(_message.Message):
    __slots__ = ("count",)
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...

class ListOfEntryRequest(_message.Message):
    __slots__ = ("logEntryRequest",)
    LOGENTRYREQUEST_FIELD_NUMBER: _ClassVar[int]
    logEntryRequest: _containers.RepeatedCompositeFieldContainer[LogEntryRequest]
    def __init__(
        self,
        logEntryRequest: _Optional[_Iterable[_Union[LogEntryRequest, _Mapping]]] = ...,
    ) -> None: ...

class ListOfActionLogRequest(_message.Message):
    __slots__ = ("actionLogRequest",)
    ACTIONLOGREQUEST_FIELD_NUMBER: _ClassVar[int]
    actionLogRequest: _containers.RepeatedCompositeFieldContainer[ActionLogRequest]
    def __init__(
        self,
        actionLogRequest: _Optional[
            _Iterable[_Union[ActionLogRequest, _Mapping]]
        ] = ...,
    ) -> None: ...

class ActionLogRequest(_message.Message):
    __slots__ = ("timestamp", "json_param")
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    JSON_PARAM_FIELD_NUMBER: _ClassVar[int]
    timestamp: int
    json_param: str
    def __init__(
        self, timestamp: _Optional[int] = ..., json_param: _Optional[str] = ...
    ) -> None: ...

class LogEntryRequest(_message.Message):
    __slots__ = ("timestamp", "category", "app_name", "app_prop", "log_entry")
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    CATEGORY_FIELD_NUMBER: _ClassVar[int]
    APP_NAME_FIELD_NUMBER: _ClassVar[int]
    APP_PROP_FIELD_NUMBER: _ClassVar[int]
    LOG_ENTRY_FIELD_NUMBER: _ClassVar[int]
    timestamp: int
    category: str
    app_name: str
    app_prop: str
    log_entry: LogEntry
    def __init__(
        self,
        timestamp: _Optional[int] = ...,
        category: _Optional[str] = ...,
        app_name: _Optional[str] = ...,
        app_prop: _Optional[str] = ...,
        log_entry: _Optional[_Union[LogEntry, _Mapping]] = ...,
    ) -> None: ...

class LogEntryResult(_message.Message):
    __slots__ = ("error", "error_message")
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ERROR_MESSAGE_FIELD_NUMBER: _ClassVar[int]
    error: int
    error_message: str
    def __init__(
        self, error: _Optional[int] = ..., error_message: _Optional[str] = ...
    ) -> None: ...

class LogEntry(_message.Message):
    __slots__ = (
        "ip",
        "src_id",
        "des_id",
        "command",
        "sub_command",
        "result",
        "start_time",
        "execute_time",
        "json_param",
        "json_ext_param",
        "client_type",
        "client_version",
        "type",
        "dp_log",
        "dp_cate",
    )
    IP_FIELD_NUMBER: _ClassVar[int]
    SRC_ID_FIELD_NUMBER: _ClassVar[int]
    DES_ID_FIELD_NUMBER: _ClassVar[int]
    COMMAND_FIELD_NUMBER: _ClassVar[int]
    SUB_COMMAND_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    START_TIME_FIELD_NUMBER: _ClassVar[int]
    EXECUTE_TIME_FIELD_NUMBER: _ClassVar[int]
    JSON_PARAM_FIELD_NUMBER: _ClassVar[int]
    JSON_EXT_PARAM_FIELD_NUMBER: _ClassVar[int]
    CLIENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    CLIENT_VERSION_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    DP_LOG_FIELD_NUMBER: _ClassVar[int]
    DP_CATE_FIELD_NUMBER: _ClassVar[int]
    ip: str
    src_id: int
    des_id: int
    command: int
    sub_command: int
    result: int
    start_time: int
    execute_time: int
    json_param: str
    json_ext_param: str
    client_type: int
    client_version: int
    type: int
    dp_log: str
    dp_cate: str
    def __init__(
        self,
        ip: _Optional[str] = ...,
        src_id: _Optional[int] = ...,
        des_id: _Optional[int] = ...,
        command: _Optional[int] = ...,
        sub_command: _Optional[int] = ...,
        result: _Optional[int] = ...,
        start_time: _Optional[int] = ...,
        execute_time: _Optional[int] = ...,
        json_param: _Optional[str] = ...,
        json_ext_param: _Optional[str] = ...,
        client_type: _Optional[int] = ...,
        client_version: _Optional[int] = ...,
        type: _Optional[int] = ...,
        dp_log: _Optional[str] = ...,
        dp_cate: _Optional[str] = ...,
    ) -> None: ...



import sys

from logging import Handler, LogRecord

from kilmlogger.services.scribe.client import ScribeClient


class GRPCEventStreamingHandler(Handler):
    def __init__(self, client: ScribeClient = ScribeClient()):
        """
        Initialize the handler.

        If stream is not specified, sys.stderr is used.
        """
        Handler.__init__(self)
        self._client = client

    def emit(self, record: LogRecord):
        self.acquire()
        try:
            self._client.log(
                msg=record.msg,
            )
        except Exception:
            sys.stdout.write("--- Logging error ---\n")
        finally:
            self.release()



from typing import TYPE_CHECKING

from starlette.datastructures import MutableHeaders
from asgi_correlation_id import CorrelationIdMiddleware as ASGICorrelationIdMiddleware

from kilmlogger.constants import CORRELATION_HEADER


if TYPE_CHECKING:
    from starlette.types import Receive, Scope, Send


class CorrelationIdMiddleware(ASGICorrelationIdMiddleware):
    header_name: str = CORRELATION_HEADER
    is_required_header: bool = True

    def __init__(self, app, **kwargs):
        if "header_name" in kwargs:
            self.header_name = kwargs.pop("header_name")
        if "is_required_header" in kwargs:
            self.is_required_header = kwargs.pop("is_required_header")

        super().__init__(app, header_name=self.header_name)

    async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
        """
        Check header
        """
        # Try to load correlation ID from the request headers
        headers = MutableHeaders(scope=scope)
        header_value = headers.get(self.header_name.lower())
        if not header_value and self.is_required_header:
            raise ValueError(f"Missing {self.header_name} header")

        return await super().__call__(scope, receive, send)



import logging

import structlog

from structlog.dev import (
    Column,
    KeyValueColumnFormatter,
    BRIGHT,
    DIM,
    CYAN,
    RESET_ALL,
    RED,
    YELLOW,
    GREEN,
    RED_BACK,
)

from kilmlogger.constants import (
    TIME_FORMAT,
    DEFAULT_FILE_LOG,
    REPLACE_KEY_FOR_TIMESTAMP,
    REPLACE_KEY_FOR_EVENT,
    TIME_FIELD_NAME,
    MESSAGE_FIELD_NAME,
    LEVEL_FIELD_NAME,
    CORRELATION_FIELD_NAME,
)
from kilmlogger.utils import add_correlation
from kilmlogger.styles import LogLevelColumnFormatter


LOG_LEVEL_COLORS = {
    "critical": RED,
    "exception": RED,
    "error": RED,
    "warn": YELLOW,
    "warning": YELLOW,
    "info": GREEN,
    "debug": GREEN,
    "notset": RED_BACK,
}


def load_default_configuration(logging_filename: str = DEFAULT_FILE_LOG) -> None:
    # Configure logging
    logging.config.dictConfig(
        {
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {
                "colored": {
                    "()": structlog.stdlib.ProcessorFormatter,
                    "processors": [
                        structlog.stdlib.ProcessorFormatter.remove_processors_meta,
                        structlog.dev.ConsoleRenderer(
                            colors=True,
                            columns=[
                                Column(
                                    "",
                                    KeyValueColumnFormatter(
                                        key_style=None,
                                        value_style=DIM,
                                        reset_style=RESET_ALL,
                                        value_repr=str,
                                    ),
                                ),
                                Column(
                                    TIME_FIELD_NAME,
                                    KeyValueColumnFormatter(
                                        key_style=None,
                                        value_style=RESET_ALL,
                                        reset_style=RESET_ALL,
                                        value_repr=str,
                                    ),
                                ),
                                Column(
                                    LEVEL_FIELD_NAME,
                                    LogLevelColumnFormatter(
                                        LOG_LEVEL_COLORS, reset_style=RESET_ALL
                                    ),
                                ),
                                Column(
                                    CORRELATION_FIELD_NAME,
                                    KeyValueColumnFormatter(
                                        key_style=None,
                                        value_style=CYAN,
                                        reset_style=RESET_ALL,
                                        value_repr=str,
                                        width=30,
                                    ),
                                ),
                                Column(
                                    MESSAGE_FIELD_NAME,
                                    KeyValueColumnFormatter(
                                        key_style=None,
                                        value_style=BRIGHT,
                                        reset_style=RESET_ALL,
                                        value_repr=str,
                                        width=30,
                                    ),
                                ),
                            ],
                        ),
                    ],
                },
            },
            "handlers": {
                "default": {
                    "level": "INFO",
                    "class": "logging.StreamHandler",
                    "formatter": "colored",
                },
                "file": {
                    "level": "INFO",
                    "class": "logging.handlers.WatchedFileHandler",
                    "filename": logging_filename,
                },
                "scribe": {
                    "level": "INFO",
                    "class": "kilmlogger.handler.GRPCEventStreamingHandler",
                },
            },
            "loggers": {
                "": {
                    "handlers": ["default", "file", "scribe"],
                    "level": "INFO",
                    "propagate": True,
                },
            },
        }
    )

    # Configure structlog
    structlog.configure(
        processors=[
            structlog.stdlib.filter_by_level,
            structlog.stdlib.add_log_level,
            add_correlation,
            structlog.processors.TimeStamper(
                fmt=TIME_FORMAT, key=REPLACE_KEY_FOR_TIMESTAMP
            ),
            structlog.processors.StackInfoRenderer(),
            structlog.processors.format_exc_info,
            structlog.processors.EventRenamer(to=REPLACE_KEY_FOR_EVENT),
            structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
        ],
        wrapper_class=structlog.stdlib.BoundLogger,
        logger_factory=structlog.stdlib.LoggerFactory(),
        cache_logger_on_first_use=True,
    )



import logging

from typing import Any

from asgi_correlation_id import correlation_id

from kilmlogger.constants import CORRELATION_KEY


def add_correlation(
    logger: logging.Logger, method_name: str, event_dict: dict[str, Any]
) -> dict[str, Any]:
    """Add request id to log message."""
    if request_id := correlation_id.get():
        event_dict[CORRELATION_KEY] = request_id
    return event_dict



CORRELATION_KEY = "correlation_id"
CORRELATION_HEADER = "X-Correlation-ID"

TIME_FORMAT = "%Y-%m-%d %H:%M:%f %Z"
REPLACE_KEY_FOR_TIMESTAMP = "time"
REPLACE_KEY_FOR_EVENT = "msg"

LEVEL_FIELD_NAME = "level"
CORRELATION_FIELD_NAME = "correlation_id"
TIME_FIELD_NAME = "time"
MESSAGE_FIELD_NAME = "msg"

DEFAULT_FILE_LOG = "logs/kilmlogger.log"



from typing import Any, Sequence

from structlog.typing import WrappedLogger, EventDict


class OrderedDictProcessor:
    def __init__(
        self,
        key_order: Sequence[str] | None = None,
        drop_missing: bool = False,
    ):
        self.key_order = key_order
        self.drop_missing = drop_missing

    def __call__(self, _: WrappedLogger, __: str, event_dict: EventDict) -> str:
        def ordered_items(event_dict: EventDict) -> list[tuple[str, Any]]:
            items = {}
            for key in self.key_order:
                value = event_dict.pop(key, None)
                if value is not None or not self.drop_missing:
                    items.update(
                        {
                            key: value,
                        }
                    )
            return items

        return ordered_items(event_dict)
